@@ -1,11 +1,11 @@
 #define NAME          "IPhreeqc"
 #define VER_MAJOR      3
 #define VER_MINOR      7
-#define VER_PATCH      4
-#define VER_REVISION   16082
-#define GIT_COMMIT     f5cb62067c371d11e1cf9a04170ea342fea8300d
+#define VER_PATCH      5
+#define VER_REVISION   16379
+#define GIT_COMMIT     78e93f68cbab7b61521b50b47c82bfc9b3e77809
 
-#define RELEASE_DATE           "March 26, 2022"
+#define RELEASE_DATE           "January 17, 2023"
 
 #define APR_STRINGIFY(n) APR_STRINGIFY_HELPER(n)
 #define APR_STRINGIFY_HELPER(n) #n
