@@ -13,6 +13,14 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 PHRQ_io::
 PHRQ_io(void)
 {
