@@ -481,7 +481,7 @@ public:
 
 	// parse.cpp -------------------------------
 	int check_eqn(int association);
-	int get_charge(char* charge, LDBLE* z);
+	int get_charge(char* charge, size_t charge_size, LDBLE* z);
 	int get_elt(const char** t_ptr, std::string& element, int* i);
 	int get_elts_in_species(const char** t_ptr, LDBLE coef);
 	int get_num(const char** t_ptr, LDBLE* num);
