@@ -16,6 +16,15 @@ typedef unsigned char boolean;
 #define OPTION_ERROR -3
 #define OPTION_DEFAULT -4
 #define OPT_1 -5
+
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
 read_solution_spread(void)
@@ -34,13 +43,13 @@ read_solution_spread(void)
  *         ERROR   if error occurred reading data
  *
  */
-	struct spread_row *heading, *row_ptr, *units;
+	class spread_row *heading, *row_ptr, *units;
 	int count, strings, numbers;
 	int spread_lines;
-	char *ptr;
-	struct defaults soln_defaults;
+	const char* cptr;
+	class defaults soln_defaults;
 	int return_value, opt;
-	char *next_char;
+	const char* next_char;
 	const char *opt_list[] = {
 		"temp",					/* 0 */
 		"temperature",			/* 1 */
@@ -79,14 +88,16 @@ read_solution_spread(void)
 
 
 	/* fill in soln_defaults.iso */
-	soln_defaults.count_iso = count_iso_defaults;
-	soln_defaults.iso =	(struct iso *) PHRQ_malloc((size_t) soln_defaults.count_iso *
-								   sizeof(struct iso));
-	if (soln_defaults.iso == NULL)
-		malloc_error();
+	soln_defaults.iso.resize(count_iso_defaults);
+
 	/* all iso[i].name is hsave'd, so no conflicts */
-	memcpy(soln_defaults.iso, iso_defaults,
-		   (size_t) soln_defaults.count_iso * sizeof(struct iso));
+	// memcpy(&soln_defaults.iso[0], iso_defaults,
+	// 	   soln_defaults.iso.size() * sizeof(class iso));
+	for (size_t i = 0; i < count_iso_defaults; ++i) {
+		soln_defaults.iso[i].name        = iso_defaults[i].name;
+		soln_defaults.iso[i].value       = iso_defaults[i].value;
+		soln_defaults.iso[i].uncertainty = iso_defaults[i].uncertainty;
+	}
 
 	heading = NULL;
 	units = NULL;
@@ -103,10 +114,10 @@ read_solution_spread(void)
 		if (spread_lines == 0 && opt != OPTION_DEFAULT)
 		{
 			row_ptr = string_to_spread_row(line);
-			ptr = line;
+			cptr = line;
 			count = numbers = strings = 0;
 			int j;
-			while (((j = copy_token(token, &ptr)) != EMPTY))
+			while (((j = copy_token(token, &cptr)) != EMPTY))
 			{
 				count++;
 				if (j == UPPER || j == LOWER)
@@ -117,14 +128,16 @@ read_solution_spread(void)
 			/*
 			 * Is 2nd token all number
 			 */
-			ptr = line;
-			copy_token(token, &ptr);
-			j = copy_token(token, &ptr);
+			cptr = line;
+			copy_token(token, &cptr);
+			j = copy_token(token, &cptr);
 			bool num = false;
 			if (j == DIGIT)
 			{
-				strtod(token.c_str(), &ptr);
-				int j1 = copy_token(token1, &ptr);
+				char* ptr;
+				(void)strtod(token.c_str(), &ptr);
+				cptr = ptr;
+				int j1 = copy_token(token1, &cptr);
 				if (j1 != EMPTY)
 				{
 					num = FALSE;
@@ -134,12 +147,11 @@ read_solution_spread(void)
 					num = TRUE;
 				}
 			}
-
 			/*
 			 *   Starts with hyphen
 			 */
-			ptr = line;
-			copy_token(token, &ptr);
+			cptr = line;
+			copy_token(token, &cptr);
 			if (token[0] == '-')
 			{
 				/* opt = opt; */
@@ -210,7 +222,7 @@ read_solution_spread(void)
 					break;
 				case 14: /* pressure */
 				case 15: /* press */
-					sscanf(next_char, SCANFORMAT, &(soln_defaults.pressure));
+					(void)sscanf(next_char, SCANFORMAT, &(soln_defaults.pressure));
 					break;
 				}
 			}
@@ -269,7 +281,7 @@ read_solution_spread(void)
 			break;
 		case 0:				/* temperature */
 		case 1:
-			sscanf(next_char, SCANFORMAT, &(soln_defaults.temp));
+			(void)sscanf(next_char, SCANFORMAT, &(soln_defaults.temp));
 			break;
 		case 2:				/* density */
 		case 3:
@@ -331,7 +343,7 @@ read_solution_spread(void)
 			break;
 		case 6:				/* ph */
 			copy_token(token, &next_char);
-			sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.ph));
+			(void)sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.ph));
 			if (copy_token(token, &next_char) != EMPTY)
 			{
 				warning_msg
@@ -340,7 +352,7 @@ read_solution_spread(void)
 			break;
 		case 7:				/* pe */
 			copy_token(token, &next_char);
-			sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.pe));
+			(void)sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.pe));
 			if (copy_token(token, &next_char) != EMPTY)
 			{
 				warning_msg
@@ -367,32 +379,20 @@ read_solution_spread(void)
 					error_msg(error_string, PHRQ_io::OT_CONTINUE);
 					continue;
 				}
-				int i;
-				for (i = 0; i < soln_defaults.count_iso; i++)
+				size_t i;
+				for (i = 0; i < soln_defaults.iso.size(); i++)
 				{
 					if (strcmp(token.c_str(), soln_defaults.iso[i].name) == 0)
 					{
 						break;
 					}
 				}
-				if (i == soln_defaults.count_iso)
+				if (i == soln_defaults.iso.size())
 				{
-					soln_defaults.iso =
-						(struct iso *) PHRQ_realloc(soln_defaults.iso,
-						(size_t) (i +
-						1) *
-						sizeof(struct iso));
-					if (soln_defaults.iso == NULL)
-					{
-						malloc_error();
-					}
-					else
-					{
-						soln_defaults.iso[i].name = string_hsave(token.c_str());
-						soln_defaults.iso[i].value = NAN;
-						soln_defaults.iso[i].uncertainty = NAN;
-						soln_defaults.count_iso++;
-					}
+					soln_defaults.iso.resize((size_t)i + 1);
+					soln_defaults.iso[i].name = string_hsave(token.c_str());
+					soln_defaults.iso[i].value = NAN;
+					soln_defaults.iso[i].uncertainty = NAN;
 				}
 
 				/* read and store isotope ratio uncertainty */
@@ -409,7 +409,7 @@ read_solution_spread(void)
 					}
 					else
 					{
-						sscanf(token.c_str(), SCANFORMAT,
+						(void)sscanf(token.c_str(), SCANFORMAT,
 							&(soln_defaults.iso[i].uncertainty));
 					}
 				}
@@ -431,7 +431,7 @@ read_solution_spread(void)
 				}
 				else
 				{
-					sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.water));
+					(void)sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.water));
 				}
 			}
 			break;
@@ -453,32 +453,20 @@ read_solution_spread(void)
 					error_msg(error_string, PHRQ_io::OT_CONTINUE);
 					continue;
 				}
-				int i;
-				for (i = 0; i < soln_defaults.count_iso; i++)
+				size_t i;
+				for (i = 0; i < soln_defaults.iso.size(); i++)
 				{
 					if (strcmp(token.c_str(), soln_defaults.iso[i].name) == 0)
 					{
 						break;
 					}
 				}
-				if (i == soln_defaults.count_iso)
+				if (i == soln_defaults.iso.size())
 				{
-					soln_defaults.iso =
-						(struct iso *) PHRQ_realloc(soln_defaults.iso,
-						(size_t) (i +
-						1) *
-						sizeof(struct iso));
-					if (soln_defaults.iso == NULL)
-					{
-						malloc_error();
-					}
-					else
-					{
-						soln_defaults.iso[i].name = string_hsave(token.c_str());
-						soln_defaults.iso[i].value = NAN;
-						soln_defaults.iso[i].uncertainty = NAN;
-						soln_defaults.count_iso++;
-					}
+					soln_defaults.iso.resize(i + 1);
+					soln_defaults.iso[i].name = string_hsave(token.c_str());
+					soln_defaults.iso[i].value = NAN;
+					soln_defaults.iso[i].uncertainty = NAN;
 				}
 				/* read and store isotope ratio */
 				if (copy_token(token, &next_char) != DIGIT)
@@ -489,7 +477,7 @@ read_solution_spread(void)
 					error_msg(error_string, CONTINUE);
 					break;
 				}
-				sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.iso[i].value));
+				(void)sscanf(token.c_str(), SCANFORMAT, &(soln_defaults.iso[i].value));
 				/* read and store isotope ratio uncertainty */
 				int j;
 				if ((j = copy_token(token, &next_char)) != EMPTY)
@@ -504,14 +492,14 @@ read_solution_spread(void)
 					}
 					else
 					{
-						sscanf(token.c_str(), SCANFORMAT,
+						(void)sscanf(token.c_str(), SCANFORMAT,
 							&(soln_defaults.iso[i].uncertainty));
 					}
 				}
 			}
 			break;
 		case 14: /* pressure */
-			sscanf(next_char, SCANFORMAT, &(soln_defaults.pressure));
+			(void)sscanf(next_char, SCANFORMAT, &(soln_defaults.pressure));
 			break;
 		case 100:				/* read headings */
 			heading = string_to_spread_row(line);
@@ -519,8 +507,8 @@ read_solution_spread(void)
 				int i;
 				for (i = 0; i < heading->count; i++)
 				{
-					while (replace(" ", "", heading->char_vector[i]) == TRUE);
-					while (replace(",", "_", heading->char_vector[i]) == TRUE);
+					while (replace(" ", "", heading->str_vector[i]) == TRUE);
+					while (replace(",", "_", heading->str_vector[i]) == TRUE);
 				}
 			}
 			break;
@@ -539,18 +527,18 @@ read_solution_spread(void)
 		assert(g_spread_sheet.units == NULL);
 		g_spread_sheet.units = copy_row(units);
 	}
-	copy_defaults(&g_spread_sheet.defaults, &soln_defaults);
+	g_spread_sheet.defaults = soln_defaults;
 #endif
 	spread_row_free(heading);
 	spread_row_free(units);
 
-	soln_defaults.iso = (struct iso *) free_check_null(soln_defaults.iso);
+	soln_defaults.iso.clear();
 	return (return_value);
 }
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
-spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
-					   struct spread_row *data, struct defaults defaults)
+spread_row_to_solution(class spread_row *heading, class spread_row *units,
+					   class spread_row *data, class defaults defaults)
 /* ---------------------------------------------------------------------- */
 {
 	Keywords::KEYWORDS next_keyword_save;
@@ -560,7 +548,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 	CParser parser(this->phrq_io);
 
 	int return_value, opt;
-	char *next_char;
+	const char* next_char;
 	const char *opt_list[] = {
 		"temp",					/* 0 */
 		"temperature",			/* 1 */
@@ -590,7 +578,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 		int i; 
 		for (i = 0; i < heading->count; i++)
 		{
-			if (strcmp_nocase(heading->char_vector[i], "number") == 0)
+			if (strcmp_nocase(heading->str_vector[i].c_str(), "number") == 0)
 			{
 				break;
 			}
@@ -605,13 +593,13 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 			input_error++;
 			error_string = sformatf(
 				"Expected solution number or number range in 'number' column, found:  %s.",
-				data->char_vector[i]);
+				data->str_vector[i].c_str());
 			error_msg(error_string, CONTINUE);
 		}
 		else
 		{
 			string = "solution_s ";
-			string.append( data->char_vector[i] );
+			string.append(data->str_vector[i]);
 			next_keyword_save = next_keyword;
 			next_keyword = Keywords::KEY_SOLUTION_SPREAD;
 			cxxNumKeyword nk;
@@ -655,7 +643,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 	initial_data_ptr->Set_units(defaults.units);
 	initial_data_ptr->Set_default_pe(defaults.redox);
 	{
-		cxxChemRxn temp_chem_reaction;
+		CReaction temp_chem_reaction;
 		initial_data_ptr->Get_pe_reactions()[defaults.redox] = temp_chem_reaction;
 	}
 /*
@@ -664,13 +652,13 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 	return_value = UNKNOWN;
 	for (int i = 0; i < heading->count; i++)
 	{
-		if (strcmp_nocase(heading->char_vector[i], "number") == 0)
+		if (strcmp_nocase(heading->str_vector[i].c_str(), "number") == 0)
 			continue;
-		if (strcmp_nocase(heading->char_vector[i], "uncertainty") == 0)
+		if (strcmp_nocase(heading->str_vector[i].c_str(), "uncertainty") == 0)
 			continue;
-		if (strcmp_nocase(heading->char_vector[i], "uncertainties") == 0)
+		if (strcmp_nocase(heading->str_vector[i].c_str(), "uncertainties") == 0)
 			continue;
-		if (strcmp_nocase(heading->char_vector[i], "isotope_uncertainty") ==
+		if (strcmp_nocase(heading->str_vector[i].c_str(), "isotope_uncertainty") ==
 			0)
 			continue;
 		/*
@@ -678,14 +666,14 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 		 */
 		if (heading->type_vector[i] == EMPTY)
 			continue;
-		string = heading->char_vector[i];
+		string = heading->str_vector[i];
 		string.append(" ");
 		/*
 		 *  Copy in concentration data
 		 */
 		if (i >= data->count || data->type_vector[i] == EMPTY)
 			continue;
-		string.append(data->char_vector[i]); 
+		string.append(data->str_vector[i]);
 		string.append(" ");
 		/*
 		 *  Copy in concentration data
@@ -693,7 +681,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 		if (units != NULL && i < units->count
 			&& units->type_vector[i] != EMPTY)
 		{
-			string.append(units->char_vector[i]);
+			string.append(units->str_vector[i]);
 		}
 /*
  *   Parse string just like read_solution input 
@@ -720,14 +708,14 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 			break;
 		case 0:				/* temperature */
 		case 1:
-			sscanf(next_char, SCANFORMAT, &dummy);
+			(void)sscanf(next_char, SCANFORMAT, &dummy);
 			temp_solution.Set_tc(dummy);
 			break;
 		case 2:				/* density */
 		case 3:
 			{
 				int j = copy_token(token, &next_char);
-				sscanf(token.c_str(), SCANFORMAT, &dummy);
+				(void)sscanf(token.c_str(), SCANFORMAT, &dummy);
 				temp_solution.Set_density(dummy);
 				j = copy_token(token, &next_char);
 				if (j != EMPTY)
@@ -767,7 +755,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 			{
 				const char * pe_str = string_hsave(token.c_str());
 				initial_data_ptr->Set_default_pe(pe_str);
-				cxxChemRxn temp_chem_reaction;
+				CReaction temp_chem_reaction;
 				initial_data_ptr->Get_pe_reactions()[token] = temp_chem_reaction;
 			}
 			else
@@ -831,7 +819,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 					error_msg(error_string, PHRQ_io::OT_CONTINUE);
 					error_string = sformatf( "\t%s\t%s\n", "line_save:   ", line_save);
 					error_msg(error_string, PHRQ_io::OT_CONTINUE);
-//struct spread_row
+//class spread_row
 //{
 //	int count;
 //	int empty, string, number;
@@ -845,7 +833,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 					{
 						for (int ii = 0; ii < heading->count; ii++)
 						{
-							error_string = sformatf("%d\t%s\n",ii,heading->char_vector[ii]);
+							error_string = sformatf("%d\t%s\n",ii,heading->str_vector[ii].c_str());
 							error_msg(error_string, PHRQ_io::OT_CONTINUE);
 						}
 					}
@@ -860,7 +848,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 					{
 						for (int ii = 0; ii < data->count; ii++)
 						{
-							error_string = sformatf("%d\t%s\t%d\n",ii,data->char_vector[ii],data->type_vector[ii]);
+							error_string = sformatf("%d\t%s\t%d\n",ii,data->str_vector[ii].c_str(),data->type_vector[ii]);
 							error_msg(error_string, PHRQ_io::OT_CONTINUE);
 						}
 					}
@@ -874,7 +862,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 					{
 						for (int ii = 0; ii < units->count; ii++)
 						{
-							error_string = sformatf("%d\t%s\n",ii,units->char_vector[ii]);
+							error_string = sformatf("%d\t%s\n",ii,units->str_vector[ii].c_str());
 							error_msg(error_string, PHRQ_io::OT_CONTINUE);
 						}
 					}
@@ -890,10 +878,10 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 				/* read and save element name */
 				{
 					char *temp_iso_name = string_duplicate(token.c_str());
-					char *ptr1 = temp_iso_name;
-					get_num(&ptr1, &dummy);
+					const char* cptr1 = temp_iso_name;
+					get_num(&cptr1, &dummy);
 					temp_isotope.Set_isotope_number(dummy);
-					if (ptr1[0] == '\0' || isupper((int) ptr1[0]) == FALSE)
+					if (cptr1[0] == '\0' || isupper((int)cptr1[0]) == FALSE)
 					{
 						error_msg("Expecting element name.", PHRQ_io::OT_CONTINUE);
 						error_msg(line_save, PHRQ_io::OT_CONTINUE);
@@ -902,7 +890,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 						char_string = (char*)free_check_null(char_string);
 						return (CParser::PARSER_ERROR);
 					}
-					temp_isotope.Set_elt_name(ptr1);
+					temp_isotope.Set_elt_name(cptr1);
 					temp_iso_name = (char*)free_check_null(temp_iso_name);
 				}
 				/* read and store isotope ratio */
@@ -915,7 +903,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 					free_check_null(char_string);
 					continue;
 				}
-				sscanf(token.c_str(), SCANFORMAT, &dummy);
+				(void)sscanf(token.c_str(), SCANFORMAT, &dummy);
 				temp_isotope.Set_ratio(dummy);
 				temp_isotope.Set_ratio_uncertainty(NAN);
 
@@ -932,7 +920,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 						free_check_null(char_string);
 						continue;
 					}
-					sscanf(token.c_str(), SCANFORMAT, &dummy);
+					(void)sscanf(token.c_str(), SCANFORMAT, &dummy);
 					temp_isotope.Set_ratio_uncertainty(dummy);
 				}
 				temp_solution.Get_isotopes()[temp_isotope.Get_isotope_name()] = temp_isotope;
@@ -940,8 +928,6 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 			break;
 		case 10:				/* water */
 			{
-				//next_char = char_string;
-				//int j = copy_token(token, &next_char); // read identifier "water"
 				int j = copy_token(token, &next_char);
 				if (j == EMPTY)
 				{
@@ -956,7 +942,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 				}
 				else
 				{
-					sscanf(token.c_str(), SCANFORMAT, &dummy);
+					(void)sscanf(token.c_str(), SCANFORMAT, &dummy);
 					temp_solution.Set_mass_water(dummy);
 				}
 			}
@@ -1007,7 +993,7 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 				initial_data_ptr->Get_comps()[temp_comp.Get_description()] = temp_comp;
 				if (temp_comp.Get_pe_reaction().size() > 0)
 				{
-					cxxChemRxn temp_chem_reaction;
+					CReaction temp_chem_reaction;
 					initial_data_ptr->Get_pe_reactions()[temp_comp.Get_pe_reaction()] = temp_chem_reaction;
 				}
 			}
@@ -1060,206 +1046,93 @@ spread_row_to_solution(struct spread_row *heading, struct spread_row *units,
 	return (return_value);
 }
 /* ---------------------------------------------------------------------- */
-struct spread_row * Phreeqc::
+class spread_row * Phreeqc::
 string_to_spread_row(char *string)
 /* ---------------------------------------------------------------------- */
 {
-	int j, l;
-	/* possible memory error if length of line is smaller than previous line */
-	char *token;
-	char *ptr;
-	struct spread_row *spread_row_ptr = NULL;
+	int j;
+	std::string token;
+	const char* cptr;
 /*
  *   Allocate space
  */
-	token = (char *) PHRQ_malloc(strlen(line) + 1);
-	if (token == NULL)
-	{
-		malloc_error();
-		return spread_row_ptr;
-	}
-	spread_row_ptr =
-		(struct spread_row *) PHRQ_malloc((size_t) sizeof(struct spread_row));
+	class spread_row* spread_row_ptr = new class spread_row;
 	if (spread_row_ptr == NULL)
 	{
 		malloc_error();
 		return spread_row_ptr;
 	}
-	spread_row_ptr->char_vector =
-		(char **) PHRQ_malloc((size_t) spread_length * sizeof(char *));
-	if (spread_row_ptr->char_vector == NULL)
-	{
-		malloc_error();
-		return spread_row_ptr;
-	}
-	spread_row_ptr->d_vector =
-		(LDBLE *) PHRQ_malloc((size_t) spread_length * sizeof(LDBLE));
-	if (spread_row_ptr->d_vector == NULL)
-	{
-		malloc_error();
-		return spread_row_ptr;
-	}
-	spread_row_ptr->type_vector =
-		(int *) PHRQ_malloc((size_t) spread_length * sizeof(int));
-	if (spread_row_ptr->type_vector == NULL)
-	{
-		malloc_error();
-		return spread_row_ptr;
-	}
 	spread_row_ptr->count = 0;
 	spread_row_ptr->empty = 0;
 	spread_row_ptr->string = 0;
 	spread_row_ptr->number = 0;
-	ptr = string;
+	cptr = string;
 /*
  *   Split by tabs, reallocate space
  */
 	for (;;)
 	{
-		if (spread_row_ptr->count + 1 > spread_length)
-		{
-			spread_length *= 2;
-
-			spread_row_ptr->char_vector =
-				(char **) PHRQ_realloc(spread_row_ptr->char_vector,
-									   (size_t) spread_length * sizeof(char *));
-			if (spread_row_ptr->char_vector == NULL)
-			{
-				malloc_error();
-				return spread_row_ptr;
-			}
-
-			spread_row_ptr->d_vector =
-				(LDBLE *) PHRQ_realloc(spread_row_ptr->d_vector,
-									   (size_t) spread_length * sizeof(LDBLE));
-			if (spread_row_ptr->d_vector == NULL)
-			{
-				malloc_error();
-				return spread_row_ptr;
-			}
-
-			spread_row_ptr->type_vector =
-				(int *) PHRQ_realloc(spread_row_ptr->type_vector,
-									 (size_t) spread_length * sizeof(int));
-			if (spread_row_ptr->type_vector == NULL)
-			{
-				malloc_error();
-				return spread_row_ptr;
-			}
-		}
-		j = copy_token_tab(token, &ptr, &l);
+		j = copy_token_tab(token, &cptr);
 		if (j == EOL)
 			break;
-		spread_row_ptr->char_vector[spread_row_ptr->count] =
-			string_duplicate(token);
-		spread_row_ptr->d_vector[spread_row_ptr->count] = NAN;
-		if (j == EMPTY || l == 0)
+		spread_row_ptr->str_vector.push_back(token);
+		if (j == EMPTY || token.size() == 0)
 		{
 			spread_row_ptr->empty++;
-			spread_row_ptr->type_vector[spread_row_ptr->count] = EMPTY;
+			spread_row_ptr->type_vector.push_back(EMPTY);
 		}
 		else if (j == UPPER || j == LOWER)
 		{
 			spread_row_ptr->string++;
-			spread_row_ptr->type_vector[spread_row_ptr->count] = STRING;
+			spread_row_ptr->type_vector.push_back(STRING);
 		}
 		else if (j == DIGIT)
 		{
 			spread_row_ptr->number++;
-			spread_row_ptr->d_vector[spread_row_ptr->count] =
-				strtod(token, NULL);
-			spread_row_ptr->type_vector[spread_row_ptr->count] = NUMBER;
+			spread_row_ptr->type_vector.push_back(NUMBER);
 		}
 		else
 		{
 			input_error++;
 			error_msg("Unknown input in string_to_spread_row keyword.", CONTINUE);
-			error_string = sformatf("\tcopy_token j: %d, token: %s\n", j, token);
+			error_string = sformatf("\tcopy_token j: %d, token: %s\n", j, token.c_str());
 			error_msg(error_string, CONTINUE);
 			error_msg(line_save, CONTINUE);
 		}
 		spread_row_ptr->count++;
 	}
-/*
- *   Clean up and return
- */
-	if (spread_row_ptr->count == 0)
-	{
-		spread_row_ptr->char_vector =
-			(char **) free_check_null(spread_row_ptr->char_vector);
-		spread_row_ptr->d_vector =
-			(LDBLE *) free_check_null(spread_row_ptr->d_vector);
-		spread_row_ptr->type_vector =
-			(int *) free_check_null(spread_row_ptr->type_vector);
-	}
-	else
-	{
-/*  Do not realloc to smaller size, memory error */
-/*
-    spread_row_ptr->char_vector =
-      (char **) PHRQ_realloc (spread_row_ptr->char_vector,
-			      (size_t) spread_row_ptr->count *
-			      sizeof (char *));
-    if (spread_row_ptr->char_vector == NULL)
-      malloc_error ();
-    spread_row_ptr->d_vector =
-      (LDBLE *) PHRQ_realloc (spread_row_ptr->d_vector,
-			      (size_t) spread_row_ptr->count *
-			      sizeof (LDBLE));
-    if (spread_row_ptr->d_vector == NULL)
-      malloc_error ();
-    spread_row_ptr->type_vector =
-      (int *) PHRQ_realloc (spread_row_ptr->type_vector,
-			    (size_t) spread_row_ptr->count * sizeof (int));
-    if (spread_row_ptr->type_vector == NULL)
-      malloc_error ();
-*/
-	}
-	token = (char *) free_check_null(token);
+	assert(spread_row_ptr->count == spread_row_ptr->str_vector.size());
+	assert(spread_row_ptr->count == spread_row_ptr->type_vector.size());
+	assert(spread_row_ptr->count == spread_row_ptr->empty + spread_row_ptr->string + spread_row_ptr->number);
 	return (spread_row_ptr);
 }
 
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
-spread_row_free(struct spread_row *spread_row_ptr)
+spread_row_free(class spread_row *spread_row_ptr)
 /* ---------------------------------------------------------------------- */
 {
-	int i;
-
 	if (spread_row_ptr == NULL)
 		return (OK);
-	for (i = 0; i < spread_row_ptr->count; i++)
-	{
-		spread_row_ptr->char_vector[i] =
-			(char *) free_check_null(spread_row_ptr->char_vector[i]);
-	}
-
-	spread_row_ptr->char_vector =
-		(char **) free_check_null(spread_row_ptr->char_vector);
-	spread_row_ptr->d_vector =
-		(LDBLE *) free_check_null(spread_row_ptr->d_vector);
-	spread_row_ptr->type_vector =
-		(int *) free_check_null(spread_row_ptr->type_vector);
-	spread_row_ptr = (struct spread_row *) free_check_null(spread_row_ptr);
+	spread_row_ptr->str_vector.clear();
+	spread_row_ptr->type_vector.clear();
+	delete spread_row_ptr;
 	return (OK);
 }
 
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
-copy_token_tab(char *token_ptr, char **ptr, int *length)
+copy_token_tab(std::string& token, const char **cptr)
 /* ---------------------------------------------------------------------- */
 {
 /*
- *   Copies from **ptr to *token_ptr until first tab is encountered.
+ *   Copies from **cptr to *token until first tab is encountered.
  *
  *   Arguments:
- *      *token_ptr  output, place to store token
- *
- *     **ptr        input, character string to read token from
- *                  output, next position after token
- *
- *       length     output, length of token
+ *      *token       output, place to store token
  *
+ *     **cptr        input, character string to read token from
+ *                   output, next position after token
  *   Returns:
  *      UPPER,
  *      LOWER,
@@ -1268,13 +1141,14 @@ copy_token_tab(char *token_ptr, char **ptr, int *length)
  *      EOL,
  *      UNKNOWN.
  */
-	int i, j, return_value;
+	int i, return_value;
 	char c;
 /*
  *   Strip leading spaces
  */
-	while ((c = **ptr) == ' ')
-		(*ptr)++;
+	token.clear();
+	while ((c = **cptr) == ' ')
+		(*cptr)++;
 /*
  *   Check what we have
  */
@@ -1309,10 +1183,10 @@ copy_token_tab(char *token_ptr, char **ptr, int *length)
 	i = 0;
 	for (;;)
 	{
-		c = **ptr;
+		c = **cptr;
 		if (c == '\t')
 		{
-			(*ptr)++;
+			(*cptr)++;
 			break;
 		}
 		else if (c == '\0')
@@ -1321,32 +1195,17 @@ copy_token_tab(char *token_ptr, char **ptr, int *length)
 		}
 		else
 		{
-			token_ptr[i] = c;
-			(*ptr)++;
+			token.push_back(c);
+			(*cptr)++;
 			i++;
 		}
 	}
-	token_ptr[i] = '\0';
-	*length = i;
-/*
- *   Strip trailing spaces
- */
-	for (j = i - 1; j >= 0; j--)
-	{
-		if (j != ' ')
-			break;
-	}
-	if (j != i - 1)
-	{
-		token_ptr[j + 1] = '\0';
-		*length = j + 1;
-	}
 	return (return_value);
 }
 
 /* ---------------------------------------------------------------------- */
  int Phreeqc::
-get_option_string(const char **opt_list, int count_opt_list, char **next_char)
+get_option_string(const char **opt_list, int count_opt_list, const char **next_char)
 /* ---------------------------------------------------------------------- */
 {
 /*
@@ -1354,7 +1213,7 @@ get_option_string(const char **opt_list, int count_opt_list, char **next_char)
  */
 	int j;
 	int opt_l, opt;
-	char *opt_ptr;
+	const char *opt_ptr;
 	char option[MAX_LENGTH];
 
 	opt_ptr = *next_char;
@@ -1393,7 +1252,7 @@ get_option_string(const char **opt_list, int count_opt_list, char **next_char)
 	return (j);
 }
 
-#ifdef PHREEQCI_GUI
+#if defined(PHREEQCI_GUI)
 /* ---------------------------------------------------------------------- */
 void Phreeqc::
 free_spread(void)
@@ -1402,124 +1261,36 @@ free_spread(void)
 	int i;
 	spread_row_free(g_spread_sheet.heading);
 	spread_row_free(g_spread_sheet.units);
-	for (i = 0; i < g_spread_sheet.count_rows; i++)
+	for (i = 0; i < g_spread_sheet.rows.size(); ++i)
 	{
 		spread_row_free(g_spread_sheet.rows[i]);
 	}
-	g_spread_sheet.rows = (spread_row**)free_check_null(g_spread_sheet.rows);
+	g_spread_sheet.rows.clear();
+	g_spread_sheet.defaults.iso.clear();
+	g_spread_sheet.defaults.redox = NULL;
+	g_spread_sheet.defaults.units = NULL;
 
-	for (i = 0; i < g_spread_sheet.defaults.count_iso; i++)
-	{
-		g_spread_sheet.defaults.iso[i].name =
-			(const char *)free_check_null((void*)g_spread_sheet.defaults.iso[i].name);
-	}
-	g_spread_sheet.defaults.iso =
-		(struct iso*)free_check_null(g_spread_sheet.defaults.iso);
-
-	g_spread_sheet.defaults.redox =
-		(const char *)free_check_null((void*)g_spread_sheet.defaults.redox);
-	g_spread_sheet.defaults.units =
-		(const char *)free_check_null((void*)g_spread_sheet.defaults.units);
-
-	g_spread_sheet.heading = 0;
-	g_spread_sheet.units = 0;
-	g_spread_sheet.count_rows = 0;
-	g_spread_sheet.defaults.count_iso = 0;
+	g_spread_sheet.heading = NULL;
+	g_spread_sheet.units   = NULL;
+	g_spread_sheet.defaults.iso.clear();
 }
 
 /* ---------------------------------------------------------------------- */
 void Phreeqc::
-add_row(struct spread_row *spread_row_ptr)
+add_row(class spread_row *spread_row_ptr)
 /* ---------------------------------------------------------------------- */
 {
-	g_spread_sheet.rows =
-		(struct spread_row **) PHRQ_realloc(g_spread_sheet.rows,
-											sizeof(struct spread_row *) *
-											(g_spread_sheet.count_rows + 1));
-	if (g_spread_sheet.rows == NULL)
-	{
-		malloc_error();
-	}
-	else
-	{
-		g_spread_sheet.rows[g_spread_sheet.count_rows++] =
-			copy_row(spread_row_ptr);
-	}
+	g_spread_sheet.rows.push_back(copy_row(spread_row_ptr));
 }
 
 /* ---------------------------------------------------------------------- */
-struct spread_row * Phreeqc::
-copy_row(struct spread_row *spread_row_ptr)
+class spread_row * Phreeqc::
+copy_row(class spread_row *spread_row_ptr)
 /* ---------------------------------------------------------------------- */
 {
-	int i;
-	struct spread_row *new_spread_row_ptr;
-/*
- *   Allocate space
- */
-	new_spread_row_ptr =
-		(struct spread_row *) PHRQ_malloc((size_t) sizeof(struct spread_row));
-	if (new_spread_row_ptr == NULL)
+	spread_row *copy = new spread_row(*spread_row_ptr);
+	if (copy == NULL)
 		malloc_error();
-	new_spread_row_ptr->char_vector =
-		(char **) PHRQ_malloc((size_t) spread_row_ptr->count *
-							  sizeof(char *));
-	if (new_spread_row_ptr->char_vector == NULL)
-		malloc_error();
-	new_spread_row_ptr->d_vector =
-		(LDBLE *) PHRQ_malloc((size_t) spread_row_ptr->count * sizeof(LDBLE));
-	if (new_spread_row_ptr->d_vector == NULL)
-		malloc_error();
-	new_spread_row_ptr->type_vector =
-		(int *) PHRQ_malloc((size_t) spread_row_ptr->count * sizeof(int));
-	if (new_spread_row_ptr->type_vector == NULL)
-		malloc_error();
-
-	for (i = 0; i < spread_row_ptr->count; i++)
-	{
-		new_spread_row_ptr->char_vector[i] =
-			string_duplicate(spread_row_ptr->char_vector[i]);
-		new_spread_row_ptr->d_vector[i] = spread_row_ptr->d_vector[i];
-		new_spread_row_ptr->type_vector[i] = spread_row_ptr->type_vector[i];
-	}
-	new_spread_row_ptr->count = spread_row_ptr->count;
-	new_spread_row_ptr->empty = spread_row_ptr->empty;
-	new_spread_row_ptr->number = spread_row_ptr->number;
-	new_spread_row_ptr->string = spread_row_ptr->string;
-
-	return new_spread_row_ptr;
+	return copy;
 }
-
-/* ---------------------------------------------------------------------- */
-void Phreeqc::
-copy_defaults(struct defaults *dest_ptr, struct defaults *src_ptr)
-/* ---------------------------------------------------------------------- */
-{
-	int i;
-	dest_ptr->count_iso = src_ptr->count_iso;
-	dest_ptr->density = src_ptr->density;
-	dest_ptr->iso =
-		(struct iso *) PHRQ_malloc(sizeof(struct iso) * src_ptr->count_iso);
-	if (dest_ptr->iso == NULL)
-	{
-		malloc_error();
-	}
-	else
-	{
-		for (i = 0; i < src_ptr->count_iso; i++)
-		{
-			dest_ptr->iso[i] = src_ptr->iso[i];
-			dest_ptr->iso[i].name = string_duplicate(src_ptr->iso[i].name);
-		}
-	}
-
-	dest_ptr->pe = src_ptr->pe;
-	dest_ptr->ph = src_ptr->ph;
-	dest_ptr->redox = string_duplicate(src_ptr->redox);
-	dest_ptr->temp = src_ptr->temp;
-	dest_ptr->units = string_duplicate(src_ptr->units);
-	dest_ptr->water = src_ptr->water;
-	dest_ptr->pressure = src_ptr->pressure;
-}
-
 #endif
